@@ -58,7 +58,7 @@ std::vector<std::pair<std::string, std::string>> dosage_writer::gen_headers(cons
       {"source", "Minimac v" + std::string(VERSION)},
       {"phasing", "full"},
       {"contig", "<ID=" + std::string(chromosome) + ">"},
-      {"INFO", "<ID=AF,Number=1,Type=Float,Description=\"Estimated Alternate Allele Frequency\">"},
+      {"INFO", "<ID=AF,Number=A,Type=Float,Description=\"Estimated Alternate Allele Frequency\">"},
       {"INFO", "<ID=MAF,Number=1,Type=Float,Description=\"Estimated Minor Allele Frequency\">"},
       {"INFO", "<ID=AVG_CS,Number=1,Type=Float,Description=\"Average Call Score\">"},
       {"INFO", "<ID=R2,Number=1,Type=Float,Description=\"Estimated Imputation Accuracy (R-square)\">"},
@@ -73,11 +73,11 @@ std::vector<std::pair<std::string, std::string>> dosage_writer::gen_headers(cons
     if (f == "GT")
       headers.emplace_back("FORMAT", "<ID=GT,Number=1,Type=String,Description=\"Genotype\">");
     else if (f == "DS")
-      headers.emplace_back("FORMAT", "<ID=DS,Number=1,Type=Float,Description=\"Estimated Alternate Allele Dosage : [P(0/1)+2*P(1/1)]\">");
+      headers.emplace_back("FORMAT", "<ID=DS,Number=A,Type=Float,Description=\"Estimated Alternate Allele Dosage : [P(0/1)+2*P(1/1)]\">");
     else if (f == "HDS")
-      headers.emplace_back("FORMAT", "<ID=HDS,Number=2,Type=Float,Description=\"Estimated Haploid Alternate Allele Dosage \">");
+      headers.emplace_back("FORMAT", "<ID=HDS,Number=.,Type=Float,Description=\"Estimated Haploid Alternate Allele Dosage \">");
     else if (f == "GP")
-      headers.emplace_back("FORMAT", "<ID=GP,Number=3,Type=Float,Description=\"Estimated Posterior Probabilities for Genotypes 0/0, 0/1 and 1/1 \">");
+      headers.emplace_back("FORMAT", "<ID=GP,Number=G,Type=Float,Description=\"Estimated Posterior Probabilities for Genotypes 0/0, 0/1 and 1/1 \">");
     else if (f == "SD")
       headers.emplace_back("FORMAT", "<ID=SD,Number=1,Type=Float,Description=\"Variance of Posterior Genotype Probabilities\">");
   }
@@ -103,7 +103,7 @@ std::vector<std::pair<std::string, std::string>> dosage_writer::gen_emp_headers(
     {"INFO", "<ID=IMPUTED,Number=0,Type=Flag,Description=\"Marker was imputed\">"},
     {"INFO", "<ID=TYPED,Number=0,Type=Flag,Description=\"Marker was genotyped\">"},
     {"FORMAT","<ID=GT,Number=1,Type=String,Description=\"Genotyped alleles from Array\">"},
-    {"FORMAT","<ID=LDS,Number=2,Type=Float,Description=\"Leave-one-out Imputed Dosage : Estimated Haploid Alternate Allele Dosage assuming site was NOT genotyped\">"}};
+    {"FORMAT","<ID=LDS,Number=.,Type=Float,Description=\"Leave-one-out Imputed Dosage : Estimated Haploid Alternate Allele Dosage assuming site was NOT genotyped\">"}};
 
   // TODO: command string
 
@@ -112,21 +112,30 @@ std::vector<std::pair<std::string, std::string>> dosage_writer::gen_emp_headers(
 
 savvy::file::format dosage_writer::format_from_filename(const std::string& filename, savvy::file::format default_format)
 {
-  if (filename.size() >= 3)
+  if (filename.size() >= 4)
   {
-    const std::string ext = filename.substr(filename.size() - 3);
-    if (ext == "sav")
+    const std::string ext = filename.substr(filename.size() - 4);
+    if (ext == ".sav")
       return savvy::file::format::sav;
-    else if (ext == "bcf")
+    else if (ext == ".bcf")
       return savvy::file::format::bcf;
-    else if (ext == "vcf")
+    else if (ext == ".vcf")
       return savvy::file::format::vcf;
   }
 
-  if (filename.size() >= 6)
+  if (filename.size() >= 5)
   {
-    const std::string ext = filename.substr(filename.size() - 6);
-    if (ext == "vcf.gz")
+    const std::string ext = filename.substr(filename.size() - 5);
+    if (ext == ".usav")
+      return savvy::file::format::sav;
+    else if (ext == ".ubcf")
+      return savvy::file::format::bcf;
+  }
+
+  if (filename.size() >= 7)
+  {
+    const std::string ext = filename.substr(filename.size() - 7);
+    if (ext == ".vcf.gz")
       return savvy::file::format::vcf;
   }
 
@@ -138,12 +147,30 @@ int dosage_writer::clevel_from_filename(const std::string& filename, int default
   if (filename.size() >= 4)
   {
     const std::string ext = filename.substr(filename.size() - 4);
-    if (ext == ".sav" || ext == ".bcf" || ext == "f.gz")
+    if (ext == ".sav")
       return 6;
-    else if (ext == "usav" || ext == "ubcf" || ext == ".vcf")
+    else if (ext == ".bcf")
+      return 6;
+    else if (ext == ".vcf")
       return 0;
   }
 
+  if (filename.size() >= 5)
+  {
+    const std::string ext = filename.substr(filename.size() - 5);
+    if (ext == ".usav")
+      return 0;
+    else if (ext == ".ubcf")
+      return 0;
+  }
+
+  if (filename.size() >= 7)
+  {
+    const std::string ext = filename.substr(filename.size() - 7);
+    if (ext == ".vcf.gz")
+      return 6;
+  }
+
   return default_clevel;
 }
 
@@ -181,6 +208,7 @@ bool dosage_writer::merge_temp_files(std::list<savvy::reader>& temp_files, std::
   std::vector<float> partial_lds;
   std::vector<std::int8_t> pasted_gt;
   std::vector<std::int8_t> partial_gt;
+  std::size_t max_ploidy = 0;
 
   int good_count = temp_files.size();
   while (good_count == temp_files.size())
@@ -194,8 +222,16 @@ bool dosage_writer::merge_temp_files(std::list<savvy::reader>& temp_files, std::
     good_count = 0;
     for (auto it = temp_files.begin(); it != temp_files.end(); ++it)
     {
-      good_count += (int)it->read(out_var).good();
+      bool read_succeeded = it->read(out_var).good();
+      good_count += (int)read_succeeded;
       out_var.get_format("HDS", partial_hds);
+
+      // verify max ploidy is consistent across all temp files
+      if (max_ploidy == 0)
+        max_ploidy = partial_hds.size() / it->samples().size();
+      if (read_succeeded && max_ploidy != (partial_hds.size() / it->samples().size()))
+        return std::cerr << "Error: max ploidy is not consistent across temp files. This should never happen. Please report." << std::endl, false;
+
       std::size_t old_size = pasted_hds.size();
       pasted_hds.resize(old_size + partial_hds.size());
       for (auto jt = partial_hds.begin(); jt != partial_hds.end(); ++jt)
@@ -292,6 +328,9 @@ bool dosage_writer::merge_temp_files(std::list<savvy::reader>& temp_files, std::
             if (good_count_emp < temp_emp_files.size())
               return std::cerr << "Error: record mismatch in empirical temp files" << std::endl, false;
 
+            if (pasted_hds.size() != pasted_gt.size() || pasted_lds.size() != pasted_gt.size())
+              return std::cerr << "Error: Merged HDS, LDS, and GT are not consistent. This should never happen. Please report." << std::endl, false;
+
             out_var_emp.set_format("GT", pasted_gt);
             out_var_emp.set_format("LDS", pasted_lds);
             emp_out_file_->write(out_var_emp);
